@@ -7,10 +7,9 @@
 # This file is part of Volatility.
 #
 # Volatility is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License Version 2 as
-# published by the Free Software Foundation.  You may not use, modify or
-# distribute this program under any other version of the GNU General
-# Public License.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # Volatility is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -63,7 +62,8 @@ class NoneObject(object):
     Instantiate with the reason for the error.
     """
     def __init__(self, reason = '', strict = False):
-        debug.debug("None object instantiated: " + reason, 2)
+        if not hasattr(sys, "frozen"):
+            debug.debug("None object instantiated: " + reason, 2)
         self.reason = reason
         self.strict = strict
         if strict:
@@ -542,6 +542,17 @@ class Pointer(NativeType):
         result = self.dereference()
         return result.m(memname)
 
+class Pointer32(Pointer):
+    def __init__(self, theType, offset, vm, target = None, **kwargs):
+        # Default to profile-endian address
+        # Sometimes we need a 32bit pointer on a 64bit system
+        NativeType.__init__(self, theType, offset, vm, format_string = "<I", **kwargs)
+
+        if theType:
+            self.target = Curry(Object, theType)
+        else:
+            self.target = target
+
 class Void(NativeType):
     def __init__(self, theType, offset, vm, **kwargs):
         # Default to profile-endian unsigned long
@@ -856,6 +867,7 @@ class Profile(object):
         # Prepopulate object_classes with base classes
         self.object_classes = {'BitField': BitField,
                                'Pointer': Pointer,
+                               'Pointer32':Pointer32,
                                'Void': Void,
                                'Array': Array,
                                'CType': CType,
@@ -1150,6 +1162,17 @@ class Profile(object):
                          name = name,
                          target = self._list_to_type(name, target, typeDict))
 
+        ## This is of the form [ 'pointer32' , [ 'foobar' ]]
+        if typeList[0] == 'pointer32':
+            try:
+                target = typeList[1]
+            except IndexError:
+                raise RuntimeError("Syntax Error in pointer type defintion for name {0}".format(name))
+
+            return Curry(Pointer32, None,
+                         name = name,
+                         target = self._list_to_type(name, target, typeDict))
+
         ## This is an array: [ 'array', count, ['foobar'] ]
         if typeList[0] == 'array':
             return Curry(Array, None,
