@@ -24,6 +24,8 @@ import volatility.plugins.linux.pslist as linux_pslist
 import volatility.plugins.linux.pidhashtable as linux_pidhashtable
 import volatility.plugins.linux.pslist_cache as linux_pslist_cache
 import volatility.plugins.linux.common as linux_common
+from volatility.renderers import TreeGrid
+from volatility.renderers.basic import Address
 
 #based off the windows version from mhl
 #
@@ -46,7 +48,12 @@ class linux_psxview(linux_common.AbstractLinuxCommand):
         return [x.obj_offset for x in linux_pslist_cache.linux_pslist_cache(self._config).calculate()]
 
     def _get_task_parents(self):
-        return [x.real_parent.v() for x in linux_pslist.linux_pslist(self._config).calculate()]
+        if self.addr_space.profile.obj_has_member("task_struct", "real_parent"): 
+            ret = [x.real_parent.v() for x in linux_pslist.linux_pslist(self._config).calculate()] 
+        else:
+            ret = [x.parent.v() for x in linux_pslist.linux_pslist(self._config).calculate()]
+
+        return ret
     
     def _get_thread_leaders(self):
         return [x.group_leader.v() for x in linux_pidhashtable.linux_pidhashtable(self._config).calculate()]
@@ -77,8 +84,29 @@ class linux_psxview(linux_common.AbstractLinuxCommand):
                     seen_offsets.append(offset)
                     yield offset, obj.Object("task_struct", offset = offset, vm = self.addr_space), ps_sources
 
-    def render_text(self, outfd, data):
+    def unified_output(self, data):
+        return TreeGrid([("Offset(V)", Address),
+                       ("Name", str),
+                       ("PID", int),
+                       ("pslist", str),
+                       ("pid_hash", str),
+                       ("kmem_cache", str),
+                       ("parents", str),
+                       ("leaders", str)],
+                        self.generator(data))
 
+    def generator(self, data):
+        for offset, process, ps_sources in data:
+            yield(0, [Address(offset),
+                str(process.comm),
+                int(process.pid),
+                str(ps_sources['pslist'].__contains__(offset)),
+                str(ps_sources['pid_hash'].__contains__(offset)),
+                str(ps_sources['kmem_cache'].__contains__(offset)),
+                str(ps_sources['parents'].__contains__(offset)),
+                str(ps_sources['thread_leaders'].__contains__(offset))])
+
+    def render_text(self, outfd, data):
         self.table_header(outfd, [('Offset(V)', '[addrpad]'),
                                   ('Name', '<20'),
                                   ('PID', '>6'),
