@@ -24,6 +24,8 @@ import volatility.plugins.common as common
 import volatility.win32.tasks as tasks
 import volatility.debug as debug
 import volatility.plugins.registry.registryapi as registryapi
+from volatility.renderers import TreeGrid
+from volatility.renderers.basic import Address
 
 #--------------------------------------------------------------------------------
 # vtypes
@@ -154,7 +156,8 @@ class _SERVICE_RECORD_RECENT(_SERVICE_RECORD_LEGACY):
     def traverse(self):
         """Generator that walks the singly-linked list"""
 
-        yield self # Include this object in the list 
+        if self.is_valid():
+            yield self # Include this object in the list 
 
         # Make sure we dereference these pointers, or the 
         # is_valid() checks will apply to the pointer and 
@@ -238,6 +241,7 @@ class ServiceVistax86(obj.ProfileModification):
             'Type' : [ 0x20, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
             'State': [ 0x24, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
             'Start' : [ 0x3C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
+            'ServiceList' : [ 0x5C, ['_SERVICE_LIST_ENTRY']],
             }]})
 
 class ServiceVistax64(obj.ProfileModification):
@@ -260,6 +264,7 @@ class ServiceVistax64(obj.ProfileModification):
             'Type' : [ 0x30, ['Flags', {'bitmap': SERVICE_TYPE_FLAGS}]],
             'State': [ 0x34, ['Enumeration', dict(target = 'long', choices = SERVICE_STATE_ENUM)]],
             'Start' : [ 0x4C, ['Enumeration', dict(target = 'long', choices = SERVICE_START_ENUM)]],
+            'ServiceList' : [ 0x78, ['_SERVICE_LIST_ENTRY']],
             }]})
 
 class Service8x64(obj.ProfileModification):
@@ -351,7 +356,7 @@ class SvcScan(common.AbstractWindowsCommand):
             if process_space == None:
                 continue
             # Find all instances of the record tag 
-            for address in task.search_process_memory([tag]):
+            for address in task.search_process_memory([tag], vad_filter = lambda x: x.Length < 0x40000000):
                 if version <= (5, 2):
                     # Windows XP/2003
                     rec = obj.Object("_SERVICE_RECORD", offset = address -
@@ -423,23 +428,91 @@ class SvcScan(common.AbstractWindowsCommand):
         outfd.write("}\n")
 
     @staticmethod
-    def get_service_dlls(regapi):
+    def get_service_info(regapi):
         ccs = regapi.reg_get_currentcontrolset()
         key_name = "{0}\\services".format(ccs)
-        dlls = {}
+        info = {}
         for subkey in regapi.reg_get_all_subkeys(hive_name = "system", key = key_name):
+
+            path_value = ""
+            dll_value = ""
+
+            image_path = regapi.reg_get_value(hive_name = "system", key = "", value = "ImagePath", given_root = subkey)
+            if image_path:
+                path_value = utils.remove_unprintable(image_path)
+
             for rootkey in regapi.reg_get_all_subkeys(hive_name = "system", key = "", given_root = subkey):
                 if rootkey.Name == "Parameters":
                     service_dll = regapi.reg_get_value(hive_name = "system", key = "", value = "ServiceDll", given_root = rootkey)
                     if service_dll != None:
-                        dlls[utils.remove_unprintable(str(subkey.Name))] = "{0}".format(utils.remove_unprintable(service_dll))
-        return dlls
+                        dll_value = utils.remove_unprintable(service_dll)
+                    break
+
+            info[utils.remove_unprintable(str(subkey.Name))] = (dll_value, path_value)
+
+        return info
+
+    def unified_output(self, data):
+        if self._config.VERBOSE:
+            return TreeGrid([("Offset", Address),
+                       ("Order", int),
+                       ("Start", str),
+                       ("PID", int),
+                       ("ServiceName", str),
+                       ("DisplayName", str),
+                       ("ServiceType", str),
+                       ("State", str),
+                       ("BinaryPath", str),
+                       ("ServiceDll", str),
+                       ("ImagePath", str)],
+                        self.generator(data)) 
+
+        return TreeGrid([("Offset", Address),
+                       ("Order", int),
+                       ("Start", str),
+                       ("PID", int),
+                       ("ServiceName", str),
+                       ("DisplayName", str),
+                       ("ServiceType", str),
+                       ("State", str),
+                       ("BinaryPath", str)],
+                        self.generator(data))
+
+    def generator(self, data):
+        if self._config.VERBOSE:
+            regapi = registryapi.RegistryApi(self._config)
+            info = self.get_service_info(regapi)
+
+        for rec in data:
+            if self._config.VERBOSE:
+                vals = info.get("{0}".format(rec.ServiceName.dereference()), None)
+                yield (0, [Address(rec.obj_offset),
+                        int(rec.Order),
+                        str(rec.Start),
+                        int(rec.Pid),
+                        str(rec.ServiceName.dereference() or ""),
+                        str(rec.DisplayName.dereference() or ""),
+                        str(rec.Type),
+                        str(rec.State),
+                        str(rec.Binary or ""),
+                        str(val[0] if vals else ""),
+                        str(val[1] if vals else "")])  
+            else:
+                yield (0, [Address(rec.obj_offset),
+                        int(rec.Order),
+                        str(rec.Start),
+                        int(rec.Pid),
+                        str(rec.ServiceName.dereference() or ""),
+                        str(rec.DisplayName.dereference() or ""),
+                        str(rec.Type),
+                        str(rec.State),
+                        str(rec.Binary or "")])
 
     def render_text(self, outfd, data):
 
         if self._config.VERBOSE:
             regapi = registryapi.RegistryApi(self._config)
-            dlls = self.get_service_dlls(regapi)
+            info = self.get_service_info(regapi)
 
         for rec in data:
             # This can't possibly look neat in a table with columns...
@@ -454,8 +527,9 @@ class SvcScan(common.AbstractWindowsCommand):
             outfd.write("Binary Path: {0}\n".format(rec.Binary))
 
             if self._config.VERBOSE:
-                val = dlls.get("{0}".format(rec.ServiceName.dereference()), None)
-                if val is not None:
-                    outfd.write("ServiceDll: {0}\n".format(val))
+                vals = info.get("{0}".format(rec.ServiceName.dereference()), None)
+                if vals:
+                    outfd.write("ServiceDll: {0}\n".format(vals[0]))
+                    outfd.write("ImagePath: {0}\n".format(vals[1]))
 
             outfd.write("\n")
