@@ -31,6 +31,7 @@ import volatility.constants as constants
 import volatility.plugins.overlays.native_types as native_types
 import volatility.utils as utils
 import volatility.timefmt as timefmt
+import encodings.utf_16
 
 class String(obj.BaseObject):
     """Class for dealing with Strings"""
@@ -156,10 +157,10 @@ class IpAddress(obj.NativeType):
     """Provides proper output for IpAddress objects"""
 
     def __init__(self, theType, offset, vm, **kwargs):
-        obj.NativeType.__init__(self, theType, offset, vm, format_string = vm.profile.native_types['unsigned int'][1], **kwargs)
+        obj.NativeType.__init__(self, theType, offset, vm, format_string = "4s", **kwargs)
 
     def v(self):
-        return utils.inet_ntop(socket.AF_INET, struct.pack("<I", obj.NativeType.v(self)))
+        return utils.inet_ntop(socket.AF_INET, obj.NativeType.v(self))
 
 class Ipv6Address(obj.NativeType):
     """Provides proper output for Ipv6Address objects"""
@@ -210,7 +211,9 @@ class VolatilityDTB(obj.VolatilityMagic):
 
     def generate_suggestions(self):
         offset = 0
-        data = self.obj_vm.read(offset, constants.SCAN_BLOCKSIZE)
+        data = self.obj_vm.zread(offset, constants.SCAN_BLOCKSIZE)
+        last_range_start, last_range_size = sorted(self.obj_vm.get_available_addresses())[-1]
+        max_offset = last_range_start + last_range_size
         while data:
             found = data.find(str(self.obj_parent.DTBSignature), 0)
             while found >= 0:
@@ -221,7 +224,9 @@ class VolatilityDTB(obj.VolatilityMagic):
                 found = data.find(str(self.obj_parent.DTBSignature), found + 1)
 
             offset += len(data)
-            data = self.obj_vm.read(offset, constants.SCAN_BLOCKSIZE)
+            if offset >= max_offset:
+                break 
+            data = self.obj_vm.zread(offset, constants.SCAN_BLOCKSIZE)
 
 class UnixTimeStamp(obj.NativeType):
     """Class for handling Unix Time Stamps"""
@@ -256,6 +261,22 @@ class UnixTimeStamp(obj.NativeType):
             return format(timefmt.display_datetime(dt), formatspec)
         return "-"
 
+class VolatilityMaxAddress(obj.VolatilityMagic):
+    """The maximum address of a profile's 
+    underlying AS. 
+
+    On x86 this is 0xFFFFFFFF (2 ** 32) - 1
+    On x64 this is 0xFFFFFFFFFFFFFFFF (2 ** 64) - 1 
+
+    We use a VolatilityMagic to calculate this 
+    based on the size of an address, since that's 
+    something we can already rely on being set
+    properly for the AS. 
+    """
+
+    def generate_suggestions(self):
+        yield 2 ** (self.obj_vm.profile.get_obj_size("address") * 8) - 1
+
 class BasicObjectClasses(obj.ProfileModification):
 
     def modification(self, profile):
@@ -266,8 +287,12 @@ class BasicObjectClasses(obj.ProfileModification):
             'VOLATILITY_MAGIC': VOLATILITY_MAGIC,
             'VolatilityDTB': VolatilityDTB,
             'UnixTimeStamp': UnixTimeStamp,
+            'VolatilityMaxAddress': VolatilityMaxAddress,
             })
 
+        profile.merge_overlay({'VOLATILITY_MAGIC': [None, {
+            'MaxAddress': [0x0, ['VolatilityMaxAddress']],
+            }]})
 
 ### DEPRECATED FEATURES ###
 #
