@@ -23,6 +23,7 @@ import sys, os
 import zipfile
 import struct
 import time
+from operator import attrgetter
 import volatility.plugins as plugins
 import volatility.debug as debug
 import volatility.obj as obj
@@ -195,6 +196,15 @@ class VolatilityMacIntelValidAS(obj.VolatilityMagic):
         else:
             yield False
 
+class ifnet(obj.CType):
+    def sockaddr_dl(self):
+        if hasattr(self, "if_lladdr"):
+            ret = obj.Object("sockaddr_dl", offset = self.if_lladdr.ifa_addr.v(), vm = self.obj_vm)
+        else:
+            ret = obj.Object("sockaddr_dl", offset = self.if_addrhead.tqh_first.ifa_addr.v(), vm = self.obj_vm)
+
+        return ret
+
 class vnode(obj.CType):
     def is_dir(self):
         return self.v_type == 2
@@ -245,10 +255,11 @@ class vnode(obj.CType):
         phys_as = utils.load_as(self.obj_vm.get_config(), astype = 'physical')
         
         while cur and cur.is_valid() and cur.offset < file_size:
-            buf = phys_as.zread(cur.phys_page * 4096, 4096)              
+            if cur.offset > 0:
+                buf = phys_as.zread(cur.phys_page * 4096, 4096)              
 
-            yield (cur.offset.v(), buf)
- 
+                yield (cur.offset.v(), buf)
+     
             cur = cur.listq.next.dereference_as("vm_page")
 
 class fileglob(obj.CType):
@@ -258,12 +269,50 @@ class fileglob(obj.CType):
         ret = self.members.get("fg_type")
         if ret:
             ret = self.m("fg_type")
-        else:    
-            ret = self.fg_ops.fo_type
+        else:
+            if self.fg_ops.is_valid(): 
+                ret = self.fg_ops.fo_type
+            else:
+                ret = 'INVALID'
 
         ret = str(ret)
         return ret
-        
+     
+class kauth_scope(obj.CType):
+    @property
+    def ks_identifier(self):
+        ident_ptr = self.m("ks_identifier")
+        ident = self.obj_vm.read(ident_ptr, 256)
+        if ident:
+            idx = ident.find("\x00")
+            if idx != -1:
+                ident = ident[:idx]  
+
+        return ident
+
+    def listeners(self):
+        ls_array = obj.Object(theType="Array", targetType="kauth_local_listener", offset = self.m("ks_listeners").obj_offset, vm = self.obj_vm, count = 16)    
+        for ls in ls_array:
+            if ls.is_valid() and ls.kll_callback != 0:
+                yield ls 
+
+class thread(obj.CType):   
+    def start_time(self):
+        baddr = self.obj_vm.profile.get_symbol("_clock_boottime")     
+
+        boot_time = obj.Object("unsigned long long", offset = baddr, vm = self.obj_vm)
+        abs_time  = boot_time + self.sched_stamp 
+
+        try:
+            data = struct.pack("<I", abs_time)
+        except struct.error:
+            return ""
+
+        bufferas = addrspace.BufferAddressSpace(self.obj_vm.get_config(), data = data)
+        dt = obj.Object("UnixTimeStamp", offset = 0, vm = bufferas, is_utc = True)
+
+        return dt
+
 class proc(obj.CType):   
     def __init__(self, theType, offset, vm, name = None, **kwargs):
         self.pack_fmt  = ""
@@ -281,16 +330,288 @@ class proc(obj.CType):
             self.pack_fmt = "<Q"
             self.pack_size = 8
             self.addr_type = "unsigned long long"
+      
+    def bash_hash_entries(self):
+        proc_as = self.get_process_address_space()
         
+        # In cases when mm is an invalid pointer 
+        if not proc_as:
+            return
+
+        bit_string = str(self.task.map.pmap.pm_task_map or '')[9:]
+        if bit_string.find("64BIT") == -1:
+            addr_type       = "unsigned int"
+            bucket_contents_type =  "mac32_bucket_contents"
+            htable_type     = "mac32_bash_hash_table"
+            nbuckets_offset = self.obj_vm.profile.get_obj_offset(htable_type, "nbuckets") 
+        else:
+            addr_type       = "unsigned long long"
+            bucket_contents_type =  "mac64_bucket_contents"
+            htable_type     = "mac64_bash_hash_table"
+            nbuckets_offset = self.obj_vm.profile.get_obj_offset(htable_type, "nbuckets") 
+
+        shared_start = self.task.shared_region.sr_base_address 
+        shared_end   = shared_start + self.task.shared_region.sr_size
+
+        for map in self.get_proc_maps():
+            if shared_start <= map.start <= shared_end:
+                continue
+
+            if map.get_perms() != "rw-":
+                continue
+
+            if map.get_path() != "":
+                continue
+
+            ## 1 GB limit to prevent major delays...the bash hash data
+            ## should not be found in a region larger than this. 
+            if map.end - map.start > 0x40000000:
+                continue
+
+            off = map.start
+
+            while off < map.end:
+                # test the number of buckets
+                dr = proc_as.read(off + nbuckets_offset, 4)
+                if dr == None:
+                    new_off = (off & ~0xfff) + 0xfff + 1
+                    off = new_off
+                    continue
+
+                test = struct.unpack("<I", dr)[0]
+                if test != 64:
+                    off = off + 1
+                    continue
+
+                htable = obj.Object(htable_type, offset = off, vm = proc_as)
+                
+                if htable.is_valid():
+                    bucket_array = obj.Object(theType="Array", targetType=addr_type, offset = htable.bucket_array, vm = htable.nbuckets.obj_vm, count = 64)
+
+                    for bucket_ptr in bucket_array:
+                        bucket = obj.Object(bucket_contents_type, offset = bucket_ptr, vm = htable.nbuckets.obj_vm)
+                        while bucket != None and bucket.times_found > 0:  
+                            pdata = bucket.data 
+
+                            if pdata == None:
+                                bucket = bucket.next_bucket()
+                                continue
+
+                            if pdata.is_valid() and (0 <= pdata.flags <= 2):
+                                yield bucket
+
+                            bucket = bucket.next_bucket()
+                
+                off = off + 1
+
+    def bash_history_entries(self):
+        proc_as = self.get_process_address_space()
+            
+        bit_string = str(self.task.map.pmap.pm_task_map or '')[9:]
+        if bit_string.find("64BIT") == -1:
+            pack_format = "<I"
+            hist_struct = "bash32_hist_entry"
+        else:
+            pack_format = "<Q"
+            hist_struct = "bash64_hist_entry"
+
+        # Brute force the history list of an address isn't provided 
+        ts_offset = proc_as.profile.get_obj_offset(hist_struct, "timestamp")
+
+        history_entries = [] 
+        bang_addrs = []
+
+        # Look for strings that begin with pound/hash on the process heap 
+        for ptr_hash in self.search_process_memory_rw_nofile(["#"]):                 
+            # Find pointers to this strings address, also on the heap 
+            addr = struct.pack(pack_format, ptr_hash)
+            bang_addrs.append(addr)
+
+        for (idx, ptr_string) in enumerate(self.search_process_memory_rw_nofile(bang_addrs)):
+            # Check if we found a valid history entry object 
+            hist = obj.Object(hist_struct, 
+                              offset = ptr_string - ts_offset, 
+                              vm = proc_as)
+
+            if hist.is_valid():
+                history_entries.append(hist)
+    
+        # Report everything we found in order
+        for hist in sorted(history_entries, key = attrgetter('time_as_integer')):
+            yield hist              
+
+    def _dynamic_env(self, proc_as, pack_format, addr_sz):
+        for mapping in self.get_proc_maps():
+            if not str(mapping.get_perms()) == "rw-" or mapping.get_path().find("bash") == -1:
+                continue
+
+            env_start = 0
+            for off in range(mapping.links.start, mapping.links.end):
+                # check the first index
+                addrstr = proc_as.read(off, self.pack_size)
+                if not addrstr or len(addrstr) != self.pack_size:
+                    continue
+                addr = struct.unpack(self.pack_fmt, addrstr)[0]
+                # check first idx...
+                if addr:
+                    firstaddrstr = proc_as.read(addr, self.pack_size)
+                    if not firstaddrstr or len(firstaddrstr) != self.pack_size:
+                        continue
+                    firstaddr = struct.unpack(self.pack_fmt, firstaddrstr)[0]
+                    buf = proc_as.read(firstaddr, 64)
+                    if not buf:
+                        continue
+                    eqidx = buf.find("=")
+                    if eqidx > 0:
+                        nullidx = buf.find("\x00")
+                        # single char name, =
+                        if nullidx >= eqidx:
+                            env_start = addr
+
+            if env_start == 0:
+                continue
+
+            envars = obj.Object(theType="Array", targetType=self.addr_type, vm=proc_as, offset=env_start, count=256)
+            for var in envars:
+                if var:
+                    sizes = [8, 16, 32, 64, 128, 256, 384, 512, 1024, 2048, 4096]
+                    good_varstr = None
+
+                    for size in sizes:
+                        varstr = proc_as.read(var, size)
+                        if not varstr:
+                            continue
+
+                        eqidx = varstr.find("=")
+                        idx = varstr.find("\x00")
+
+                        if idx == -1 or eqidx == -1 or idx < eqidx:
+                            continue
+                    
+                        good_varstr = varstr
+                        break
+                
+                    if good_varstr:        
+                        good_varstr = good_varstr[:idx]
+                        key = good_varstr[:eqidx]
+                        val = good_varstr[eqidx+1:]
+
+                        yield (key, val) 
+                    else:
+                        break         
+
+    def _shell_variables(self, proc_as, pack_format, addr_sz, htable_type):
+        bash_was_last = False
+        
+        nbuckets_offset = self.obj_vm.profile.get_obj_offset(htable_type, "nbuckets") 
+
+        if addr_sz == 4:
+            skip_sz = 20
+            edata_type = "mac32_envdata"
+        else:
+            skip_sz = 32
+            edata_type = "mac64_envdata"
+
+        seen = {}
+
+        for vma in self.get_proc_maps():
+            if vma.get_perms() != "rw-" or vma.get_path().endswith("/bin/bash"):
+                continue
+            
+            off = vma.start
+            end = vma.end
+            while off < end:
+                ptr_test = proc_as.read(off, addr_sz)
+                if not ptr_test:
+                    off = ((off & ~0xfff) | 0xfff) + 1
+                    continue    
+                    
+                off = off + 1
+
+                ptr = struct.unpack(pack_format, ptr_test)[0]
+              
+                if ptr in seen:
+                    continue
+                else:
+                    seen[ptr] = 1
+ 
+                ptr_test2 = proc_as.read(ptr + skip_sz, addr_sz)
+                if not ptr_test2:
+                    continue
+
+                ptr2 = struct.unpack(pack_format, ptr_test2)[0]
+
+                test = proc_as.read(ptr2 + addr_sz, 4)
+                if not test or test != "\x40\x00\x00\x00":
+                    continue
+
+                htable = obj.Object(htable_type, offset = ptr2, vm = proc_as)
+            
+                for ent in htable:
+                    key = str(ent.key)    
+                    val_addr = ent.data.dereference_as(edata_type).value
+                    if val_addr.is_valid():
+                        val = str(val_addr.dereference())
+                    else:
+                        val = ""
+
+                    yield key, val
+
+            bash_was_last = False
+
+    def bash_environment(self):
+        proc_as = self.get_process_address_space()
+        
+        # In cases when mm is an invalid pointer 
+        if not proc_as:
+            return
+
+        # Are we dealing with 32 or 64-bit pointers
+        if self.obj_vm.profile.metadata.get('memory_model', '32bit') == '32bit':
+            pack_format = "<I"
+            addr_sz = 4
+            htable_type = "mac32_bash_hash_table"
+        else:
+            pack_format = "<Q"
+            addr_sz = 8
+            htable_type = "mac64_bash_hash_table"
+
+        for key, val in self._dynamic_env(proc_as, pack_format, addr_sz):
+            yield key, val        
+
+        for key, val in self._shell_variables(proc_as, pack_format, addr_sz, htable_type):
+            yield key, val
+
+    def netstat(self):
+        for (filp, _, _) in self.lsof():
+            if filp.f_fglob.fg_type == 'DTYPE_SOCKET':
+                socket = filp.f_fglob.fg_data.dereference_as("socket") 
+                family = socket.family
+    
+                if family == 1:
+                    upcb = socket.so_pcb.dereference_as("unpcb")
+                    path = upcb.unp_addr.sun_path
+                    yield (family,  (socket.v(), path))
+                elif family in [2, 30]:
+                    proto = socket.protocol
+                    state = socket.state
+                   
+                    (lip, lport, rip, rport) = socket.get_connection_info()
+ 
+                    yield (family, (socket, proto, lip, lport, rip, rport, state))
+
     @property
     def p_gid(self):
         cred = self.p_ucred
 
         if not cred.is_valid():
-            return "-"
+            return -1
 
         if hasattr(cred, "cr_posix"):
-            ret = cred.cr_posix.cr_groups[0]
+            try:
+                ret = cred.cr_posix.cr_groups[0]
+            except IndexError:
+                ret = obj.Object("unsigned int", offset = cred.cr_posix.cr_groups.obj_offset, vm = self.obj_vm)
         else:
             ret = cred.cr_groups[0]     
     
@@ -301,7 +622,7 @@ class proc(obj.CType):
         cred = self.p_ucred
 
         if not cred.is_valid():
-            return "-"
+            return -1 
 
         if hasattr(cred, "cr_posix"):
             ret = cred.cr_posix.cr_uid
@@ -309,9 +630,20 @@ class proc(obj.CType):
             ret = cred.cr_uid     
         
         return ret
-    
-    def get_process_address_space(self):
 
+    def threads(self):
+        threads = []
+        seen_threads = []
+        qentry = self.task.threads
+        for thread in qentry.thread_walk_list(qentry.obj_offset):
+            if thread.obj_offset in seen_threads:
+                break
+            seen_threads.append(thread.obj_offset)
+            threads.append(thread)
+
+        return threads 
+
+    def get_process_address_space(self):
         cr3 = self.task.map.pmap.pm_cr3
         map_val = str(self.task.map.pmap.pm_task_map or '')
 
@@ -378,9 +710,72 @@ class proc(obj.CType):
 
         return dt
     
-    def get_dyld_maps(self):        
+    def text_start(self):
+        text_start = 0
+
+        wanted_vnode = self.p_textvp.v()
+
+        if wanted_vnode:
+            for map in self.get_proc_maps():
+                vnode = map.get_vnode()
+
+                if vnode and vnode != "sub_map" and vnode.v() == wanted_vnode:
+                    text_start = map.start.v()
+                    break
+
+        # both offset and vp were bogus
+        if text_start == 0:
+            found_map = None
+            for map in self.get_dyld_maps():
+                found_map = map
+                break
+        
+            if found_map:
+                text_start = found_map.imageLoadAddress
+
+        return text_start
+
+    def get_macho(self, exe_address):
         proc_as = self.get_process_address_space()
 
+        m = obj.Object("macho_header", offset = exe_address, vm = proc_as)
+
+        buffer = ""
+
+        for seg in m.segments():
+            if str(seg.segname) == "__PAGEZERO":
+                continue
+                
+            # this is related to the shared cache map 
+            # contact Andrew for full details
+            if str(seg.segname) == "__LINKEDIT" and seg.vmsize > 20000000:
+                continue
+
+            cur = seg.vmaddr
+            end = seg.vmaddr + seg.vmsize
+        
+            while cur < end:
+                buffer = buffer + proc_as.zread(cur, 4096) 
+                cur = cur + 4096
+ 
+        return buffer
+
+    def procdump(self):
+        start = self.text_start()
+
+        if start:
+            ret = self.get_macho(start) 
+        else:
+            ret = ""
+
+        return ret
+
+    def get_dyld_maps(self):        
+        proc_as = self.get_process_address_space()
+    
+        if proc_as == None:
+            return
+    
         if self.pack_size == 4:
             dtype = "dyld32_all_image_infos"
             itype = "dyld32_image_info"
@@ -389,6 +784,8 @@ class proc(obj.CType):
             itype = "dyld64_image_info"
 
         infos = obj.Object(dtype, offset=self.task.all_image_info_addr, vm=proc_as)
+        if not infos:
+            return
 
         # the pointer address
         info_buf = proc_as.read(infos.infoArray.obj_offset, self.pack_size)
@@ -400,8 +797,8 @@ class proc(obj.CType):
         img_infos = obj.Object(theType = "Array", targetType = itype, offset = info_addr, count = infos.infoArrayCount, vm = proc_as)
         
         for info_addr in img_infos:
-            yield info_addr
-            #yield obj.Object("dyld_image_info", offset = info_addr, vm = proc_as)   
+            if info_addr:
+                yield info_addr
 
     def get_proc_maps(self):
         map = self.task.map.hdr.links.next
@@ -412,6 +809,37 @@ class proc(obj.CType):
             yield map
             map = map.links.next
 
+    def find_heap_map(self):
+        ret = None
+
+        for pmap in self.get_proc_maps():
+            if pmap.get_special_path() == "[heap]":
+                ret = pmap
+                break
+
+        return None
+
+    def find_map(self, addr):
+        ret = None
+
+        for vma in self.get_proc_maps():
+            if int(vma.links.start) <= int(addr) <= int(vma.links.end):
+                ret = vma
+                break
+
+        return ret
+
+    def find_map_path(self, addr):
+        path = ""
+        m = self.find_map(addr)
+
+        if m:
+            path = m.get_path()
+            if path == "":
+                path = m.get_special_path()
+
+        return path
+              
     def search_process_memory(self, s):
         """Search process memory. 
 
@@ -470,6 +898,41 @@ class proc(obj.CType):
                         yield offset + hit
                 offset += min(to_read, scan_blk_sz)
 
+    def psenv(self):
+        proc_as = self.get_process_address_space()
+
+        # We need a valid process AS to continue 
+        if not proc_as:
+            return
+
+        start = self.user_stack - self.p_argslen
+        skip  = len(self.get_arguments())
+        end   = self.p_argslen
+
+        to_read = end - skip
+    
+        vars_buf = proc_as.read(start + skip, to_read)
+        if vars_buf:
+            ents = vars_buf.split("\x00")
+            for varstr in ents:
+                eqidx = varstr.find("=")
+
+                if eqidx == -1:
+                    continue
+
+                key = varstr[:eqidx]
+                val = varstr[eqidx+1:]
+
+                yield (key, val) 
+    
+    def get_environment(self):
+        env = ""
+
+        for (k, v) in self.psenv():
+            env = env + "{0}={1} ".format(k, v)
+
+        return env
+
     def get_arguments(self):
         proc_as = self.get_process_address_space()
 
@@ -511,6 +974,26 @@ class proc(obj.CType):
             argc -= 1            
 
         return " ".join([str(s) for s in args])
+    
+    def lsof(self):
+        num_fds = self.p_fd.fd_lastfile
+        nfiles  = self.p_fd.fd_nfiles
+        if nfiles > num_fds:
+            num_fds = nfiles
+
+        fds = obj.Object('Array', offset = self.p_fd.fd_ofiles, vm = self.obj_vm, targetType = 'Pointer', count = num_fds)
+
+        for i, fd in enumerate(fds):
+            f = fd.dereference_as("fileproc")
+            if f:
+                ftype = f.f_fglob.fg_type
+                if ftype == 'DTYPE_VNODE': 
+                    vnode = f.f_fglob.fg_data.dereference_as("vnode")
+                    path = vnode.full_path()
+                else:
+                    path = "<%s>" % ftype.replace("DTYPE_", "").lower()
+                        
+                yield f, path, i
 
 class rtentry(obj.CType):
 
@@ -540,6 +1023,8 @@ class rtentry(obj.CType):
         else:
             ret = "N/A"
 
+        return ret
+
     @property
     def delta(self):
         if self.expire() == 0:
@@ -563,22 +1048,17 @@ class rtentry(obj.CType):
     
     @property
     def source_ip(self):
-        return self.rt_nodes[0].rn_u.rn_leaf.rn_Key.dereference_as("sockaddr").get_address()
+        try:
+            node = self.rt_nodes[0]
+        except IndexError:
+            node = obj.Object("radix_node", offset = self.rt_nodes.obj_offset, vm = self.obj_vm)
+
+        return node.rn_u.rn_leaf.rn_Key.dereference_as("sockaddr").get_address()
 
     @property
     def dest_ip(self):
         return self.rt_gateway.get_address()
 
-
-
-
-
-
- 
-
-
-    
-
 class queue_entry(obj.CType):
 
     def walk_list(self, list_head):
@@ -689,9 +1169,21 @@ class OSString(obj.CType):
         if self.string == 0:
             return ""
 
+        if self.length > 4096:
+            return ""
+
         string_object = obj.Object("String", offset = self.string, vm = self.obj_vm, length = self.length)
         return str(string_object or '')
 
+class vm_map_object(obj.CType):
+    def object(self):
+        if hasattr(self, "vm_object"):
+            ret = self.m("vm_object")
+        else:
+            ret = self.vmo_object
+
+        return ret
+
 class vm_map_entry(obj.CType):
     @property
     def start(self):
@@ -713,21 +1205,22 @@ class vm_map_entry(obj.CType):
 
         return perms
 
-
     # used to find heap, stack, etc.
     def get_special_path(self):
         # check the heap
         ret = ""
 
-        for i in [1, 2, 3, 4, 6, 7, 8, 9]:
-            if self.alias == i:
-                ret = "[heap]"
-                break
+        # TODO - figure out how to track heap regions once el cap source code is released
+        if hasattr(self, "alias"):
+            for i in [1, 2, 3, 4, 6, 7, 8, 9]:
+                if self.alias == i:
+                    ret = "[heap]"
+                    break
 
         if ret != "":
             return ret
 
-        if self.alias == 30:
+        if hasattr(self, "alias") and self.alias == 30:
             ret = "[stack]"
 
         return ret 
@@ -749,6 +1242,15 @@ class vm_map_entry(obj.CType):
                 
         return ret
 
+    @property
+    def object(self): 
+        if hasattr(self, "vme_object"):
+            ret = self.vme_object
+        else:
+            ret = self.m("object")
+
+        return ret
+
     def get_vnode(self):
         map_obj = self
 
@@ -756,7 +1258,7 @@ class vm_map_entry(obj.CType):
             return "sub_map"
 
         # find_vnode_object
-        vnode_object = map_obj.object.vm_object 
+        vnode_object = map_obj.object.object() 
 
         while vnode_object.shadow.dereference() != None:
             vnode_object = vnode_object.shadow.dereference()
@@ -771,6 +1273,20 @@ class vm_map_entry(obj.CType):
 
         return ret
 
+    def is_suspicious(self):
+        ret = False        
+
+        perms = self.get_perms()
+
+        if perms == "rwx":
+           ret = True 
+
+        elif perms == "r-x" and self.get_path() == "":
+            ret = True
+ 
+        return ret
+
+
 class inpcb(obj.CType):
     
     def get_tcp_state(self):
@@ -908,7 +1424,6 @@ class sockaddr_dl(obj.CType):
         """Get the value of the sockaddr_dl object."""
 
         ret = ""
-
         for i in xrange(self.sdl_alen):
             try:
                 e = self.sdl_data[self.sdl_nlen + i]
@@ -959,7 +1474,7 @@ class dyld32_image_info(obj.CType):
         if addr == None:
             return ""
 
-        buf = self.obj_vm.read(addr, 256)
+        buf = self.obj_vm.zread(addr, 256)
         if buf:
             idx = buf.find("\x00")
             if idx != -1:
@@ -991,7 +1506,7 @@ class dyld64_image_info(obj.CType):
         if addr == None:
             return ""
 
-        buf = self.obj_vm.read(addr, 256)
+        buf = self.obj_vm.zread(addr, 256)
         if buf:
             idx = buf.find("\x00")
             if idx != -1:
@@ -1231,32 +1746,30 @@ def MacProfileFactory(profpkg):
             
             return ret
 
-
-        def _get_symbol_by_address(self, module, sym_address):
+        def _fill_sba_cache(self):
             ret = ""
             
             symtable = self.sys_map
-
-            mod = symtable[module]
+            mod = symtable["kernel"]
 
             for (name, addrs) in mod.items():
                 for (addr, _) in addrs:
-                    key = "%s|%x" % (module, addr)
+                    key = "%s|%x" % ("kernel", addr)
+                    self.sba_cache[key] = name
+
+                    key = "%s|%x" % ("kernel", addr + self.shift_address)
                     self.sba_cache[key] = name
-         
-                    if sym_address == addr or sym_address == self.shift_address + addr:
-                        ret = name
-                        break
-                    
-            return ret
         
         def get_symbol_by_address(self, module, sym_address):
+            if self.sba_cache == {}:
+                self._fill_sba_cache()
+    
             key = "%s|%x" % (module, sym_address)
             if key in self.sba_cache:
                 ret = self.sba_cache[key]
             else:
-                ret = self._get_symbol_by_address(module, sym_address)
-            
+                ret = ""
+ 
             return ret
 
         def get_all_symbol_names(self, module = "kernel"):
@@ -1382,10 +1895,13 @@ class MacObjectClasses(obj.ProfileModification):
             'VolatilityDTB': VolatilityDTB,
             'VolatilityMacIntelValidAS' : VolatilityMacIntelValidAS,
             'proc'  : proc,
+            'thread'  : thread,
+            'kauth_scope'  : kauth_scope,
             'dyld32_image_info' : dyld32_image_info,
             'dyld64_image_info' : dyld64_image_info,
             'fileglob' : fileglob,
             'vnode' : vnode,
+            'ifnet' : ifnet,
             'socket' : socket,
             'inpcbinfo' : inpcbinfo,
             'inpcb' : inpcb,
@@ -1398,6 +1914,7 @@ class MacObjectClasses(obj.ProfileModification):
             'sockaddr' : sockaddr, 
             'sockaddr_dl' : sockaddr_dl,
             'vm_map_entry' : vm_map_entry,
+            'vm_map_object' : vm_map_object,
             'rtentry' : rtentry,
             'queue_entry' : queue_entry,
         })
@@ -1449,12 +1966,6 @@ mac_overlay = {
     'sysctl_oid' : [ None, { 
         'oid_name' : [ None, ['pointer', ['String', dict(length = 256)]]], 
         }], 
-    'dyld32_image_info' : [ None, { 
-        'imageFilePath' : [ None, ['pointer', ['String', dict(length = 256)]]], 
-        }], 
-    'dyld64_image_info' : [ None, { 
-        'imageFilePath' : [ None, ['pointer', ['String', dict(length = 256)]]], 
-        }], 
     'sockaddr_un': [ None, { 
         'sun_path' : [ None, ['String', dict(length = 104)]],
         }],
